# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Group definition for worker pools logs."""

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import flags


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Logs(base.Group):
  """Read logs for Cloud Run worker pools."""

  detailed_help = {
      'EXAMPLES': """
          To tail logs for a worker pool, run:

            $ {command} tail my-worker-pool

          To read logs for a worker pool, run:

            $ {command} read my-worker-pool

      """,
  }

  @staticmethod
  def Args(parser):
    """Adds --region arg."""
    flags.AddRegionArg(parser)
