# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""GKE Hub Policy Controller library to manipulate API proto messages."""


from typing import Any, Dict


def additional_properties_to_dict(spec):
  """Extracts the additional properties of a message.Message as a dictionary."""
  if spec is None:
    return {}
  return {prop.key: prop.value for prop in spec.additionalProperties}


def set_additional_properties(message, additional_properties: Dict[Any, Any]):
  """Sets additional properties on a message."""
  ls = [
      message.AdditionalProperty(key=key, value=value)
      for key, value in additional_properties.items()
  ]
  message.additionalProperties = ls
  return message
