# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for managing Cloud Composer environments."""


from googlecloudsdk.calliope import base


class Environments(base.Group):
  """Create and manage Cloud Composer environments.

  The {command} command group lets you create Cloud Composer environments
  containing an Apache Airflow setup. Additionally, the command group supports
  environment updates including varying number of machines used to run Airflow,
  setting Airflow configs, or installing Python dependencies used in Airflow
  DAGs. The command group can also be used to delete Composer environments.
  """

Environments.category = base.COMPOSER_CATEGORY
