"""Generated message classes for geminicloudassist version v1.

The AI-powered assistant for Google Cloud.
"""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'geminicloudassist'


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class GeminicloudassistProjectsLocationsGetRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class GeminicloudassistProjectsLocationsInvestigationsCreateRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsCreateRequest object.

  Fields:
    investigation: A Investigation resource to be passed as the request body.
    investigationId: Optional. ID to use for the investigation, which will
      become the final component of the investigation's resource name. This
      value should be 4-63 characters, and valid characters are /a-z-/. If not
      provided, the server will generate a unique ID for the investigation.
    parent: Required. Value for parent. Location must be set to "global".
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  investigation = _messages.MessageField('Investigation', 1)
  investigationId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class GeminicloudassistProjectsLocationsInvestigationsDeleteRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsDeleteRequest object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class GeminicloudassistProjectsLocationsInvestigationsGetIamPolicyRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsGetIamPolicyRequest
  object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class GeminicloudassistProjectsLocationsInvestigationsGetRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsGetRequest object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class GeminicloudassistProjectsLocationsInvestigationsListRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsListRequest object.

  Fields:
    filter: Optional. A string limiting the results that are returned.
    orderBy: Optional. A string indicating how the returned results should be
      ordered.
    pageSize: Optional. Requested number of items to return. Server may return
      fewer items than requested. If unspecified, server will pick an
      appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent to return Investigations for. Within the
      parent, the location must be set to "global".
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class GeminicloudassistProjectsLocationsInvestigationsPatchRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsPatchRequest object.

  Fields:
    investigation: A Investigation resource to be passed as the request body.
    name: Identifier. Name of the investigation, of the form: projects/{projec
      t_number}/locations/{location_id}/investigations/{investigation_id}
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Investigation resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields present in the request will be
      overwritten.
  """

  investigation = _messages.MessageField('Investigation', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class GeminicloudassistProjectsLocationsInvestigationsRevisionsDeleteRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsRevisionsDeleteRequest
  object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class GeminicloudassistProjectsLocationsInvestigationsRevisionsGetRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsRevisionsGetRequest
  object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class GeminicloudassistProjectsLocationsInvestigationsRevisionsListRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsRevisionsListRequest
  object.

  Fields:
    pageSize: Optional. Requested number of items to return. Server may return
      fewer items than requested. If unspecified, server will pick an
      appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The parent to return Investigation revisions for. Within
      the parent, the location must be set to "global".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class GeminicloudassistProjectsLocationsInvestigationsRevisionsRunRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsRevisionsRunRequest
  object.

  Fields:
    name: Required. Name of the investigation revision to run.
    runInvestigationRevisionRequest: A RunInvestigationRevisionRequest
      resource to be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  runInvestigationRevisionRequest = _messages.MessageField('RunInvestigationRevisionRequest', 2)


class GeminicloudassistProjectsLocationsInvestigationsSetIamPolicyRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsInvestigationsSetIamPolicyRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class GeminicloudassistProjectsLocationsInvestigationsTestIamPermissionsRequest(_messages.Message):
  r"""A
  GeminicloudassistProjectsLocationsInvestigationsTestIamPermissionsRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class GeminicloudassistProjectsLocationsListRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field. It is unsupported and
      is ignored unless explicitly documented otherwise. This is primarily for
      internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class GeminicloudassistProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class GeminicloudassistProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class GeminicloudassistProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class GeminicloudassistProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A GeminicloudassistProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the [ListOperationsResponse.unreachable] field. This can only be `true`
      when reading across collections e.g. when `parent` is set to
      `"projects/example/locations/-"`. This field is not by default supported
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class Interval(_messages.Message):
  r"""Represents a time interval, encoded as a Timestamp start (inclusive) and
  a Timestamp end (exclusive). The start must be less than or equal to the
  end. When the start equals the end, the interval is empty (matches no time).
  When both start and end are unspecified, the interval matches any time.

  Fields:
    endTime: Optional. Exclusive end of the interval. If specified, a
      Timestamp matching this interval will have to be before the end.
    startTime: Optional. Inclusive start of the interval. If specified, a
      Timestamp matching this interval will have to be the same or after the
      start.
  """

  endTime = _messages.StringField(1)
  startTime = _messages.StringField(2)


class Investigation(_messages.Message):
  r"""Holds the contents of a Gemini Cloud Assist Troubleshooting
  investigation.

  Enums:
    ExecutionStateValueValuesEnum: Output only. The execution state of the
      investigation.

  Messages:
    LabelsValue: Optional. User-defined labels for the investigation.
    ObservationsValue: Optional. Observations about the project state that
      comprise the contents of the investigation.
    ObserverStatusesValue: Optional. Statuses of observers which have been
      invoked in order to run the investigation.

  Fields:
    annotations: Optional. Additional annotations on the investigation.
    createTime: Output only. The time when the investigation was created.
    error: Output only. The error if the investigation run failed. This field
      will only be set if `execution_state` is
      `INVESTIGATION_EXECUTION_STATE_FAILED`.
    executionState: Output only. The execution state of the investigation.
    labels: Optional. User-defined labels for the investigation.
    name: Identifier. Name of the investigation, of the form: projects/{projec
      t_number}/locations/{location_id}/investigations/{investigation_id}
    observations: Optional. Observations about the project state that comprise
      the contents of the investigation.
    observerStatuses: Optional. Statuses of observers which have been invoked
      in order to run the investigation.
    operation: Output only. The resource name of the Run operation most
      recently performed on the investigation, of the form: projects/{project_
      number}/locations/{location_id}/operations/{operation_id}
    revision: Output only. Resource name of the investigation's current
      revision, of the form: projects/{project_number}/locations/{location_id}
      /investigations/{investigation_id}/revisions/{revision_id}
    title: Optional. Human-readable display title for the investigation.
    updateTime: Output only. The time when the investigation was last updated.
  """

  class ExecutionStateValueValuesEnum(_messages.Enum):
    r"""Output only. The execution state of the investigation.

    Values:
      INVESTIGATION_EXECUTION_STATE_UNSPECIFIED: Default value. This value is
        unused.
      INVESTIGATION_EXECUTION_STATE_RUNNING: The investigation is being
        executed.
      INVESTIGATION_EXECUTION_STATE_MODIFIED: The investigation has not yet
        been executed since it was last updated.
      INVESTIGATION_EXECUTION_STATE_FAILED: The investigation execution has
        completed, but the execution has failed.
      INVESTIGATION_EXECUTION_STATE_COMPLETED: All execution tasks have
        completed and the investigation is at rest.
    """
    INVESTIGATION_EXECUTION_STATE_UNSPECIFIED = 0
    INVESTIGATION_EXECUTION_STATE_RUNNING = 1
    INVESTIGATION_EXECUTION_STATE_MODIFIED = 2
    INVESTIGATION_EXECUTION_STATE_FAILED = 3
    INVESTIGATION_EXECUTION_STATE_COMPLETED = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for the investigation.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ObservationsValue(_messages.Message):
    r"""Optional. Observations about the project state that comprise the
    contents of the investigation.

    Messages:
      AdditionalProperty: An additional property for a ObservationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type ObservationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ObservationsValue object.

      Fields:
        key: Name of the additional property.
        value: A Observation attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('Observation', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ObserverStatusesValue(_messages.Message):
    r"""Optional. Statuses of observers which have been invoked in order to
    run the investigation.

    Messages:
      AdditionalProperty: An additional property for a ObserverStatusesValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        ObserverStatusesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ObserverStatusesValue object.

      Fields:
        key: Name of the additional property.
        value: A ObserverStatus attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('ObserverStatus', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('InvestigationAnnotations', 1)
  createTime = _messages.StringField(2)
  error = _messages.MessageField('Status', 3)
  executionState = _messages.EnumField('ExecutionStateValueValuesEnum', 4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  observations = _messages.MessageField('ObservationsValue', 7)
  observerStatuses = _messages.MessageField('ObserverStatusesValue', 8)
  operation = _messages.StringField(9)
  revision = _messages.StringField(10)
  title = _messages.StringField(11)
  updateTime = _messages.StringField(12)


class InvestigationAnnotations(_messages.Message):
  r"""Additional annotations on an Investigation.

  Messages:
    ExtrasMapValue: Optional. Additional client-specified annotations.

  Fields:
    extrasMap: Optional. Additional client-specified annotations.
    pagePath: Optional. The path of a web page from which this investigation
      was created.
    revisionLastRunInterval: Output only. The interval during which the
      investigation was last executed.
    supportCase: Optional. A support case ID associated with the
      investigation.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ExtrasMapValue(_messages.Message):
    r"""Optional. Additional client-specified annotations.

    Messages:
      AdditionalProperty: An additional property for a ExtrasMapValue object.

    Fields:
      additionalProperties: Additional properties of type ExtrasMapValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ExtrasMapValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  extrasMap = _messages.MessageField('ExtrasMapValue', 1)
  pagePath = _messages.StringField(2)
  revisionLastRunInterval = _messages.MessageField('Interval', 3)
  supportCase = _messages.StringField(4)


class InvestigationRevision(_messages.Message):
  r"""A revision holds a snapshot of the investigation at a past point in
  time. The most recent revision holds the investigation's current state,
  while previous revisions are immutable.

  Messages:
    LabelsValue: Optional. User-defined labels for the revision.

  Fields:
    createTime: Output only. The time when the revision was created.
    labels: Optional. User-defined labels for the revision.
    name: Identifier. The name of the revision resource, of the form: projects
      /{project_number}/locations/{location_id}/investigations/{investigation_
      id}/revisions/{revision_id}
    snapshot: Optional. Snapshot of the investigation contents at this
      revision.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for the revision.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  name = _messages.StringField(3)
  snapshot = _messages.MessageField('Investigation', 4)


class InvestigationRunParameters(_messages.Message):
  r"""Represents user parameters for running an investigation.

  Messages:
    AccessTokensValue: Optional. If populated, map of project to IAM token
      granting access to that project.

  Fields:
    accessTokens: Optional. If populated, map of project to IAM token granting
      access to that project.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AccessTokensValue(_messages.Message):
    r"""Optional. If populated, map of project to IAM token granting access to
    that project.

    Messages:
      AdditionalProperty: An additional property for a AccessTokensValue
        object.

    Fields:
      additionalProperties: Additional properties of type AccessTokensValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AccessTokensValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  accessTokens = _messages.MessageField('AccessTokensValue', 1)


class ListInvestigationRevisionsResponse(_messages.Message):
  r"""Message for response to listing revisions of a given Investigation

  Fields:
    nextPageToken: A token to use in subsequent requests in order to return
      the next set of results.
    revisions: The list of Investigation revisions
    unreachable: Unordered list. Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  revisions = _messages.MessageField('InvestigationRevision', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListInvestigationsResponse(_messages.Message):
  r"""Message for response to listing Investigations

  Fields:
    investigations: The list of Investigations
    nextPageToken: A token to use in subsequent requests in order to return
      the next set of results.
    unreachable: Unordered list. Locations that could not be reached.
  """

  investigations = _messages.MessageField('Investigation', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections e.g. when attempting to list all resources across all
      supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Observation(_messages.Message):
  r"""An observation represents a single fact about the state of the system
  under investigation, along with a ranking of its relevance.

  Enums:
    ObservationCompletionStateValueValuesEnum: Optional. Represents whether
      the observation is fully formed and able to be used to draw conclusions.
    ObservationTypeValueValuesEnum: Required. The type of the observation.
    ObserverTypeValueValuesEnum: Required. The type of observer that produced
      this observation.

  Messages:
    DataValue: Optional. A structured representation of the observation. This
      is not required or guaranteed to conform to any particular schema.
    DataUrlsValue: Optional. URLs pointing to evidence in support of this
      observation. Maps from a human-readable description to a URL.
    KnowledgeUrlsValue: Optional. URLs pointing to reference knowledge related
      to this observation. Maps from a human-readable description to a URL.

  Fields:
    baseObservations: Optional. The IDs of other observations that this
      observation is based on. For example, a hypothesis observation will
      record the observations that support that hypothesis.
    createTime: Output only. The time when the observation was created.
    data: Optional. A structured representation of the observation. This is
      not required or guaranteed to conform to any particular schema.
    dataUrls: Optional. URLs pointing to evidence in support of this
      observation. Maps from a human-readable description to a URL.
    id: Output only. Uniquely identifies this observation. This is always
      equal to the map key of this observation in the parent investigation.
    knowledgeUrls: Optional. URLs pointing to reference knowledge related to
      this observation. Maps from a human-readable description to a URL.
    observationCompletionState: Optional. Represents whether the observation
      is fully formed and able to be used to draw conclusions.
    observationType: Required. The type of the observation.
    observedNormalOperation: Optional. If false or unspecified, this
      observation gives information about an issue or root cause. If true,
      this observation indicates that a system was observed to be operating
      normally. Such negative results can be useful diagnostic findings, and
      this field has no correlation with the relevance score.
    observerType: Required. The type of observer that produced this
      observation.
    recommendation: Optional. Natural language or markdown text which
      describes a recommended action to remediate the root cause or further
      investigate the issue.
    relevantResources: Optional. The Google Cloud resources relevant to the
      observation. These should be fully qualified resource URIs, e.g.:
      "//compute.googleapis.com/projects/my-project/zones/us-
      central1-a/instances/my-instance"
    systemRelevanceScore: Optional. How relevant the observer perceives this
      observation to be. This is used to rank observations in generating
      hypotheses. The system outputs observations in the range [-1, 1], where
      -1 means completely irrelevant, 0 means neutral, and 1 means complete
      confidence in its relevance.
    text: Optional. Natural-language or markdown text explaining the
      observation.
    timeIntervals: Optional. When this observation occurred.
    title: Optional. Human-readable display title for the observation.
  """

  class ObservationCompletionStateValueValuesEnum(_messages.Enum):
    r"""Optional. Represents whether the observation is fully formed and able
    to be used to draw conclusions.

    Values:
      OBSERVATION_COMPLETION_STATE_UNSPECIFIED: Default value. Will be treated
        as COMPLETE.
      OBSERVATION_COMPLETION_STATE_COMPLETE: This observation is fully formed.
      OBSERVATION_COMPLETION_STATE_INCOMPLETE: This observation is missing
        some information, or needs further processing by a different observer.
        This type of observation may be discarded in future investigation
        runs.
    """
    OBSERVATION_COMPLETION_STATE_UNSPECIFIED = 0
    OBSERVATION_COMPLETION_STATE_COMPLETE = 1
    OBSERVATION_COMPLETION_STATE_INCOMPLETE = 2

  class ObservationTypeValueValuesEnum(_messages.Enum):
    r"""Required. The type of the observation.

    Values:
      OBSERVATION_TYPE_UNSPECIFIED: Default value.
      OBSERVATION_TYPE_CLOUD_LOG: The text of this observation is a log entry.
      OBSERVATION_TYPE_CLOUD_METRIC: The content of this observation is a
        metric or group of metrics.
      OBSERVATION_TYPE_CAIS_CONFIG: A config
      OBSERVATION_TYPE_CAIS_CONFIG_DIFF: A change of config
      OBSERVATION_TYPE_CLOUD_ALERT: An alert
      OBSERVATION_TYPE_CICD_EVENT: An event from a continuous integration
        system, e.g. prober failure.
      OBSERVATION_TYPE_TEXT_DESCRIPTION: Free text input, such as the initial
        user input. Can be markdown.
      OBSERVATION_TYPE_HYPOTHESIS: This is for [tentative] conclusions drawn
        by the system. These can become inputs to a subsequent revision. In
        this case the recommendations will be used, but not the remediation.
      OBSERVATION_TYPE_STRUCTURED_INPUT: Structured input, e.g. runbook
        parameters put into a form
      OBSERVATION_TYPE_COMPOSITE: Used for an observation that includes more
        than one kind of evidence, such as a runbook output.
      OBSERVATION_TYPE_OTHER: Runbook output can be 'other' if none of the
        other types apply.
      OBSERVATION_TYPE_RELATED_RESOURCES: Resources we think are related to
        the investigation.
      OBSERVATION_TYPE_LOG_THEME: A theme discovered in the logs.
      OBSERVATION_TYPE_CONFIG_ANALYSIS: Signals output that includes a config
        analyzed by LLM.
      OBSERVATION_TYPE_OUTAGE: Signals output that includes an outage from
        PSH.
      OBSERVATION_TYPE_KNOWLEDGE: Text that provides knowledge about a
        particular user's problem. For example, error catalog
        instructions/external links, RAG etc.
    """
    OBSERVATION_TYPE_UNSPECIFIED = 0
    OBSERVATION_TYPE_CLOUD_LOG = 1
    OBSERVATION_TYPE_CLOUD_METRIC = 2
    OBSERVATION_TYPE_CAIS_CONFIG = 3
    OBSERVATION_TYPE_CAIS_CONFIG_DIFF = 4
    OBSERVATION_TYPE_CLOUD_ALERT = 5
    OBSERVATION_TYPE_CICD_EVENT = 6
    OBSERVATION_TYPE_TEXT_DESCRIPTION = 7
    OBSERVATION_TYPE_HYPOTHESIS = 8
    OBSERVATION_TYPE_STRUCTURED_INPUT = 9
    OBSERVATION_TYPE_COMPOSITE = 10
    OBSERVATION_TYPE_OTHER = 11
    OBSERVATION_TYPE_RELATED_RESOURCES = 12
    OBSERVATION_TYPE_LOG_THEME = 13
    OBSERVATION_TYPE_CONFIG_ANALYSIS = 14
    OBSERVATION_TYPE_OUTAGE = 15
    OBSERVATION_TYPE_KNOWLEDGE = 16

  class ObserverTypeValueValuesEnum(_messages.Enum):
    r"""Required. The type of observer that produced this observation.

    Values:
      OBSERVER_TYPE_UNSPECIFIED: Default value. Unused.
      OBSERVER_TYPE_DIAGNOSTICS: Diagnostics have an explicit notion of root
        causes, e.g. via runbooks.
      OBSERVER_TYPE_SIGNALS: Signals is for processing that doesn't have
        explicit root causes.
      OBSERVER_TYPE_DETERMINISTIC_CODE: This is for code that depends only on
        premises. In particular, error catalog lookups.
      OBSERVER_TYPE_AI: This is for AI inferences that depend only on
        observations listed as premises.
      OBSERVER_TYPE_USER: User-input observation, including answers to
        clarifications. All user-specified observations will be coerced to
        this type.
      OBSERVER_TYPE_ALERT: An observation from an alert.
    """
    OBSERVER_TYPE_UNSPECIFIED = 0
    OBSERVER_TYPE_DIAGNOSTICS = 1
    OBSERVER_TYPE_SIGNALS = 2
    OBSERVER_TYPE_DETERMINISTIC_CODE = 3
    OBSERVER_TYPE_AI = 4
    OBSERVER_TYPE_USER = 5
    OBSERVER_TYPE_ALERT = 6

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DataUrlsValue(_messages.Message):
    r"""Optional. URLs pointing to evidence in support of this observation.
    Maps from a human-readable description to a URL.

    Messages:
      AdditionalProperty: An additional property for a DataUrlsValue object.

    Fields:
      additionalProperties: Additional properties of type DataUrlsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DataUrlsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DataValue(_messages.Message):
    r"""Optional. A structured representation of the observation. This is not
    required or guaranteed to conform to any particular schema.

    Messages:
      AdditionalProperty: An additional property for a DataValue object.

    Fields:
      additionalProperties: Properties of the object.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class KnowledgeUrlsValue(_messages.Message):
    r"""Optional. URLs pointing to reference knowledge related to this
    observation. Maps from a human-readable description to a URL.

    Messages:
      AdditionalProperty: An additional property for a KnowledgeUrlsValue
        object.

    Fields:
      additionalProperties: Additional properties of type KnowledgeUrlsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a KnowledgeUrlsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  baseObservations = _messages.StringField(1, repeated=True)
  createTime = _messages.StringField(2)
  data = _messages.MessageField('DataValue', 3)
  dataUrls = _messages.MessageField('DataUrlsValue', 4)
  id = _messages.StringField(5)
  knowledgeUrls = _messages.MessageField('KnowledgeUrlsValue', 6)
  observationCompletionState = _messages.EnumField('ObservationCompletionStateValueValuesEnum', 7)
  observationType = _messages.EnumField('ObservationTypeValueValuesEnum', 8)
  observedNormalOperation = _messages.BooleanField(9)
  observerType = _messages.EnumField('ObserverTypeValueValuesEnum', 10)
  recommendation = _messages.StringField(11)
  relevantResources = _messages.StringField(12, repeated=True)
  systemRelevanceScore = _messages.FloatField(13, variant=_messages.Variant.FLOAT)
  text = _messages.StringField(14)
  timeIntervals = _messages.MessageField('Interval', 15, repeated=True)
  title = _messages.StringField(16)


class ObserverStatus(_messages.Message):
  r"""An ObserverStatus represents the status of an observer at a particular
  point during execution of an investigation.

  Enums:
    ObserverExecutionStateValueValuesEnum: Optional. The current execution
      state of the observer.

  Fields:
    observer: Output only. Uniquely identifies this observer. This is always
      equal to the map key of this observer in the parent investigation.
    observerDisplayName: Required. Human-readable display name for the
      observer.
    observerErrors: Optional. An error that blocked an observer from making a
      particular observation.
    observerExecutionState: Optional. The current execution state of the
      observer.
    startTime: Optional. The time when the observer started its observation.
    updateComment: Optional. A status update from the observer.
    updateTime: Optional. The time when the observer last updated the
      investigation. If the `observer_execution_state` is COMPLETE or FAILED,
      this is the time when the observer finished its observation.
  """

  class ObserverExecutionStateValueValuesEnum(_messages.Enum):
    r"""Optional. The current execution state of the observer.

    Values:
      OBSERVER_EXECUTION_UNSPECIFIED: The observer state is unspecified.
      OBSERVER_EXECUTION_NOT_STARTED: The investigation has not yet started.
      OBSERVER_EXECUTION_RUNNING: The investigation is running and this
        observer is runnable or running.
      OBSERVER_EXECUTION_COMPLETE: The observer has finished without an
        internal error.
      OBSERVER_EXECUTION_FAILED: The observer tried to run but failed due to
        an error. This is specific to a component and may be rendered in the
        UI if the observation is shown, as very low priority.
      OBSERVER_EXECUTION_BLOCKED: The observer is blocked pending an input.
      OBSERVER_EXECUTION_INVESTIGATION_BLOCKED: The observer reports an error
        that blocks or severely impacts the investigation, for example CAIS or
        logging disabled. Should be rendered in the UI prominently.
      OBSERVER_EXECUTION_INVESTIGATION_DEGRADED: The observer reports an error
        that degrades the investigation, may require user to escalate or re-
        run the investigation after mitigating the cause.
    """
    OBSERVER_EXECUTION_UNSPECIFIED = 0
    OBSERVER_EXECUTION_NOT_STARTED = 1
    OBSERVER_EXECUTION_RUNNING = 2
    OBSERVER_EXECUTION_COMPLETE = 3
    OBSERVER_EXECUTION_FAILED = 4
    OBSERVER_EXECUTION_BLOCKED = 5
    OBSERVER_EXECUTION_INVESTIGATION_BLOCKED = 6
    OBSERVER_EXECUTION_INVESTIGATION_DEGRADED = 7

  observer = _messages.StringField(1)
  observerDisplayName = _messages.StringField(2)
  observerErrors = _messages.MessageField('Status', 3, repeated=True)
  observerExecutionState = _messages.EnumField('ObserverExecutionStateValueValuesEnum', 4)
  startTime = _messages.StringField(5)
  updateComment = _messages.StringField(6)
  updateTime = _messages.StringField(7)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class RunInvestigationRevisionRequest(_messages.Message):
  r"""Request for running an investigation at a particular revision.

  Fields:
    runParameters: Optional. Parameters to pass through.
  """

  runParameters = _messages.MessageField('InvestigationRunParameters', 1)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    GeminicloudassistProjectsLocationsInvestigationsGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')
