# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://biglake.googleapis.com/deltasharing/v1/'
DOCS_URL = 'https://cloud.google.com/bigquery/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  DELTASHARING_V1_PROJECTS = (
      'deltasharing.v1.projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  DELTASHARING_V1_PROJECTS_CATALOGS_SHARES = (
      'deltasharing.v1.projects.catalogs.shares',
      'projects/{projectsId}/catalogs/{catalogsId}/shares/{sharesId}',
      {},
      ['projectsId', 'catalogsId', 'sharesId'],
      True
  )
  DELTASHARING_V1_PROJECTS_CATALOGS_SHARES_SCHEMAS = (
      'deltasharing.v1.projects.catalogs.shares.schemas',
      'projects/{projectsId}/catalogs/{catalogsId}/shares/{sharesId}/schemas/'
      '{schemasId}',
      {},
      ['projectsId', 'catalogsId', 'sharesId', 'schemasId'],
      True
  )
  DELTASHARING_V1_PROJECTS_CATALOGS_SHARES_SCHEMAS_TABLES = (
      'deltasharing.v1.projects.catalogs.shares.schemas.tables',
      'projects/{projectsId}/catalogs/{catalogsId}/shares/{sharesId}/schemas/'
      '{schemasId}/tables/{tablesId}',
      {},
      ['projectsId', 'catalogsId', 'sharesId', 'schemasId', 'tablesId'],
      True
  )
  DELTASHARING_V1_PROJECTS_CATALOGS = (
      'deltasharing.v1.projects.catalogs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/catalogs/{catalogsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing
