// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package pipeline_models;

enum DeploymentOrigination {
  ORIGINATION_UNDEFINED = 0;
  GIT_CI_CD = 1;
  LOCAL_DEPLOY = 2;
}

message DeploymentDetails {
  string git_repo = 1;
  string git_branch = 2;
  string commit_sha = 3;
}

message VersionMetadata {
  DeploymentOrigination origination = 1;
  DeploymentDetails deployment_details = 2;
}

message VersionHistory {
  string timestamp = 1;
  string version_id = 2;
  repeated string pipelines = 3;
  VersionMetadata metadata = 4;
}

message Manifest {
  string bundle = 1;
  string default_version = 2;
  string updated_at = 3;
  repeated string paused_pipelines = 4;
  repeated VersionHistory versions_history = 5;
}
