# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility class for working with batch versions."""



def is_rm_batch(batch):
  """Returns true if the batch is using resource manager based on the version."""
  is_gce_spark = (
      batch.runtimeConfig.version.startswith('1')
      or batch.runtimeConfig.version.startswith('2')
  ) and batch.rayBatch is None
  return not is_gce_spark
