# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for constants in commands for Anthos GKE On-Prem clusters."""


OPERATIONS_FORMAT = """
table(
    name.basename():label=NAME,
    metadata.verb:label=ACTION,
    metadata.target.segment(-2):label=TYPE,
    metadata.target.basename():label=TARGET,
    name.segment(3):label=LOCATION,
    done:label=DONE,
    metadata.createTime.date():label=START_TIME:sort=1,
    metadata.endTime.date():label=END_TIME
)
"""
