# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Package for the machine image CLI commands."""


from googlecloudsdk.calliope import base


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class MachineImage(base.Group):
  """Read and manage Compute Engine machine image resources."""
  category = base.COMPUTE_CATEGORY

MachineImage.detailed_help = {
    'DESCRIPTION': """
        Read and manage Compute Engine machine image resources..

        For more information about machine images, see the
        [machine images documentation](https://cloud.google.com/compute/docs/machine-images).

        See also: [Machine images API](https://cloud.google.com/compute/docs/reference/rest/v1/machineImages).
    """,
}
