# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud app versions describe` command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.calliope import base


class Describe(base.DescribeCommand):
  """Display all data about an existing version."""

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--service', '-s', required=True,
        help='The service corresponding to the version to show.')
    parser.add_argument('version', help='The ID of the version to show.')

  def Run(self, args):
    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    return api_client.GetVersionResource(service=args.service,
                                         version=args.version)
