# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud run instances dev group."""

from googlecloudsdk.calliope import base


@base.UniverseCompatible
class Dev(base.Group):
  """Commands for development on Cloud Run Instances.

  This set of commands can be used for development on Cloud Run Instances.

  To start new sync session, use `{parent_command} sync`.
  """

  detailed_help = {
      'EXAMPLES': (
          """
          To start a new sync session, run:

            $ {command} sync my-instance --source=.
      """
      ),
  }
