# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating Assured Workloads data controls."""


from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.base import ReleaseTrack


@base.ReleaseTracks(ReleaseTrack.GA, ReleaseTrack.BETA, ReleaseTrack.ALPHA)
class Assured(base.Group):
  """Read and manipulate Assured Workloads data controls."""

  category = base.SECURITY_CATEGORY

  detailed_help = {
      'DESCRIPTION':
          """
        Read and manipulate Assured Workloads data controls.
    """,
  }

  def Filter(self, context, args):
    # TODO(b/190525911):  Determine if command group works with project number
    base.RequireProjectID(args)
    del context, args
